import 'package:routex_core/routex_core.dart';
import 'package:routex_web/src/route_impl.dart';
import 'package:routex_web/src/router_impl.dart';

abstract class RoutingContextImplBase extends RoutingContext {
  final Set<RouteImpl> routes;
  final HttpServerRequest _request;

  Iterator<RouteImpl> iter;
  final String _mountPoint;
  RouteImpl currentroute;
  AtomicInteger _currentRouteNextHandlerIndex;
  AtomicInteger _currentRouteNextFailureHandlerIndex;

  RoutingContextImplBase(
    this._mountPoint,
    this._request,
    this.routes,
  ) : this.iter = routes.iterator {
    _currentRouteNextHandlerIndex = new AtomicInteger();
    _currentRouteNextFailureHandlerIndex = new AtomicInteger();
  }

  String mountPoint() => _mountPoint;

  @override
  Route currentRoute() => currentroute;

  int currentRouteNextHandlerIndex() =>
      _currentRouteNextHandlerIndex.intValue();

  int currentRouteNextFailureHandlerIndex() =>
      _currentRouteNextFailureHandlerIndex.intValue();



  void restart() {
    this.iter = routes.iterator;
    currentroute = null;
    next();
  }

  @override
  HttpServerRequest request() => _request;

  Future<bool> iterateNext() async {
    bool _failed = failed();
    if (currentroute != null) {
      try {
        if ((!_failed) && currentroute.hasNextContextHandler(this)) {
          _currentRouteNextHandlerIndex.incrementAndGet();
          await currentroute.handleContext(this);
          return true;
        } else {
          if (_failed && currentroute.hasNextFailureHandler(this)) {
            _currentRouteNextFailureHandlerIndex.incrementAndGet();
            await currentroute.handleFailure(this);
            return true;
          }
        }
      } catch (error, stackTrace) {
        if (!_failed) {
          fail(error);
          print("fail(t); 1");
          print(stackTrace);
        } else {
          await unhandledFailure(-1, error, currentroute.router());
        }
        return true;
      }
    }
    while (iter.moveNext()) {
      RouteImpl route = iter.current;
      _currentRouteNextHandlerIndex.set(0);
      _currentRouteNextFailureHandlerIndex.set(0);
      try {
        if (route.matches(this, mountPoint(), _failed)) {
          try {
            currentroute = route;
            if (_failed && currentroute.hasNextFailureHandler(this)) {
              _currentRouteNextFailureHandlerIndex.incrementAndGet();
              await route.handleFailure(this);
            } else if (currentroute.hasNextContextHandler(this)) {
              _currentRouteNextHandlerIndex.incrementAndGet();
              await route.handleContext(this);
            } else {
              continue;
            }
          } catch (error,st) {
            if (!_failed) {
              fail(error);
              print(error);
              print(st);
              print("fail(t); 2");

            } else {
              await unhandledFailure(-1, error, route.router());
            }
          }
          return true;
        }
      } catch (e) {
        if (!this.response().ended())
          await unhandledFailure(
              (e is IllegalArgumentException) != null ? 400 : -1,
              e,
              route.router());
        return true;
      }
    }
    return false;
  }

  Future<void> unhandledFailure(
      int statusCode, Object failure, RouterImpl router) async {
    int code = statusCode != -1 ? statusCode : 500;

    Handler<RoutingContext> errorHandler =
        router.getErrorHandlerByStatusCode(code);
    if (errorHandler == null) {
      this.response().fail(ResponseStatusException(code, failure: failure));
    } else {
      await errorHandler.handle(this);
    }
  }
}
